

/*
    HTTP over TLS (HTTPS) example sketch

    This example demonstrates how to use
    WiFiClientSecure class to access HTTPS API.
    We fetch and display the status of
    esp8266/Arduino project continuous integration
    build.

    Created by Ivan Grokhotkov, 2015.
    This example is in public domain.
*/

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include "certs.h"



#ifndef STASSID
#define STASSID "WiFiName"                //Hier Wifi Namen Eintragen
#define STAPSK  "WifiPasswort"            //Hier WiFi Passwort eintragen
#endif

const char* ssid = STASSID;
const char* password = STAPSK;
String line ="";
unsigned long startMillis;  //some global variables available anywhere in the program
unsigned long currentMillis;    

X509List cert(cert_DigiCert_High_Assurance_EV_Root_CA);
String url = "";
const int ledPin = LED_BUILTIN;  // the number of the LED pin

//*************TIMER
#if !defined(ESP8266)
  #error This code is designed to run on ESP8266 and ESP8266-based boards! Please check your Tools->Board setting.
#endif

// These define's must be placed at the beginning before #include "ESP8266TimerInterrupt.h"
// _TIMERINTERRUPT_LOGLEVEL_ from 0 to 4
// Don't define _TIMERINTERRUPT_LOGLEVEL_ > 0. Only for special ISR debugging only. Can hang the system.
#define TIMER_INTERRUPT_DEBUG         0
#define _TIMERINTERRUPT_LOGLEVEL_     1

// Select a Timer Clock
#define USING_TIM_DIV1                false           // for shortest and most accurate timer
#define USING_TIM_DIV16               false           // for medium time and medium accurate timer
#define USING_TIM_DIV256              true            // for longest timer but least accurate. Default

#include "ESP8266TimerInterrupt.h"

//#define BUILTIN_LED     2       // Pin D4 mapped to pin GPIO2/TXD1 of ESP8266, NodeMCU and WeMoS, control on-board LED

volatile bool statusLed = false;
volatile uint32_t lastMillis = 0;
volatile uint32_t lastFeed = 0;         //Time of last Feed in msec

#define STEPPERSPEED 20       //mm/sec 
#define STEPS_PER_MM 80   //80: DRV8825, 20Tooth Pulley, GT2 Belt, 16 Mircosteps
#define STEPS_PER_SEC STEPPERSPEED*STEPS_PER_MM   //1600
#define MAX_X 800         //Max Weg X [mm]

#define TIMER_INTERVAL_MICROS  300  //round((1.000.000/(STEPS_PER_SEC*2)))  //usec   intervall = 1.000.000/STEPS_PER_SEC

// Init ESP8266 timer 1
ESP8266Timer ITimer;



/////////////////SCHRITTMOTOR X-Achse CNCV3*******************************
#define DirXPin D5       //RAMPS1.4:A1  //CNCV3: 5
#define StepXPin D2      //RAMPS1.4:A0  //CNCV3: 3
#define DirYPin D6       //RAMPS1.4:A1  //CNCV3: 5
#define StepYPin D3      //RAMPS1.4:A0  //CNCV3: 6
#define EnablePinDriverX D8  //CNCV3,4:8, RAMPS1.4: D38
#define EndstopPin D7      // endstop -> D7 ist Z- Endstop am CNCv3, leider kein Pull-Up verbaut -> 3.3 und GND an den Schalter bauen
#define MAX_STEPS_X STEPS_PER_MM*MAX_X*2    // max Steps in X, Steigende und Abfallende Flanke = 1 Step  -> x2
#define MAX_STEPS_Y 12800    // 1 Umdehung für die Fütterung, DRV8825, 1/32 microschritte -> 200 * 32 * 2 = 12800, Steigende und Abfallende Flanke = 1 Step  -> x2
#define MillsRstTimeFeed  7200*1000     //Alle 2 h


bool b_Step= LOW;
bool EndstopXMin_Triggered = false;     //1-> triggert
bool EndstopXMax_Triggered = false;
int Fuetteranzahl = 0;              // Anzahl, wie oft gefüttert wurde
int EndstopX = 0;                   //0-> keine Endstop getriggert, 1-> Min Endstop, 2-> Max Endstop
int ResetFlag = 1;                  //Default reset of Feed counter
bool futteraktiv = false;                   //Stepper für Fütterung dreht sich
unsigned long StepCounterX = 0;
unsigned long StepCounterY = 0;     //Futterautomat
unsigned long Weg = 0;              //[mm]
int XMovement = 0;          //0-> Stop, 1-> Rechts 2-> Links

//INTERRUPT =======================================================================
void IRAM_ATTR TimerHandler()
{
  EndstopXMin_Triggered = !(digitalRead(EndstopPin));                                  //Endstop X min und max auslesen, Pullup-> getriggert wenn HIGH 
  if (StepCounterX >= MAX_STEPS_X) {EndstopXMax_Triggered = true;}
  else {EndstopXMax_Triggered = false;}


  if (EndstopXMin_Triggered == 1 && (XMovement == 2)) {           //X-Endstop Min ausgelöst und in Min-X BEwegung (links)
  StepCounterX = 0;
  digitalWrite(EnablePinDriverX, HIGH);                             //Treiber abschalten
  XMovement = 0;
  } 
  if ((StepCounterX >= MAX_STEPS_X) && (XMovement == 1)){            //Software Max Endstop ausgelöst und in Rechts Bewegung
  digitalWrite(EnablePinDriverX, HIGH);
  XMovement = 0;
  }
  if (futteraktiv){                             //Fütterung aktiv
  digitalWrite(StepYPin, b_Step);               // Fütterautomat 1/2 Schritt machen
  StepCounterY++;
  if (StepCounterY>=MAX_STEPS_Y)
        {futteraktiv = false;
        StepCounterY = 0;
        Fuetteranzahl++;
        lastFeed = millis();
        digitalWrite(EnablePinDriverX, HIGH);}  //Fütterung durchgeführt für heute, Treiber deaktivieren
  }
  else {digitalWrite(StepXPin, b_Step);}        // X-Achse 1/2 Schritt machen


  //digitalWrite(StepXPin, b_Step);


  b_Step = !b_Step;
  if (XMovement == 1)
        {StepCounterX++;}
  else if (XMovement == 2 && StepCounterX>0)
        {StepCounterX--;}
  Weg = StepCounterX/(STEPS_PER_MM*2);
//#if (TIMER_INTERRUPT_DEBUG > 0){
//  Serial.println("Delta ms = " + String(millis() - lastMillis));
//  lastMillis = millis();}
//#endif
}


void setup() {
  pinMode(ledPin, OUTPUT);
  //pinMode(EndstopPin, INPUT_PULLUP);    //LED Pin needs no pullup
  pinMode(EndstopPin, INPUT);
  pinMode(DirXPin, OUTPUT);
  pinMode(StepXPin, OUTPUT);
  pinMode(DirYPin, OUTPUT);
  pinMode(StepYPin, OUTPUT);
  pinMode(EnablePinDriverX, OUTPUT);

  digitalWrite(EnablePinDriverX, HIGH);     //LOW-> Stepper Treiber an
  digitalWrite(DirXPin, LOW); 
  digitalWrite(StepXPin, LOW); 
  digitalWrite(DirYPin, LOW); 
  digitalWrite(StepYPin, LOW);    
  ITimer.attachInterruptInterval(TIMER_INTERVAL_MICROS * 1, TimerHandler);
  ////////////////////////////////////////////////////////////////////////////////////////
// Nach dem Einschalten erstmal in Home Position -> Endanschlag fahren
 while(EndstopXMin_Triggered==0){
  digitalWrite(EnablePinDriverX, LOW);    //Treiber an
  digitalWrite(DirXPin, LOW);             //Links, Richtung setzen
  XMovement = 2;

 }
  StepCounterX = 0;                        // Im Endanschlag Stepcounter nullen
  StepCounterY = 0;
///////////////////////////////////////////////////////////////////////////////////////

  Serial.begin(115200);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  // Set time via NTP, as required for x.509 validation
  //configTime(3 * 3600, 0, "pool.ntp.org", "time.nist.gov");

  //Serial.print("Waiting for NTP time sync: ");
  //time_t now = time(nullptr);
  //while (now < 8 * 3600 * 2) {
  //  delay(500);
  //  Serial.print(".");
  //  now = time(nullptr);
  //}
  //Serial.println("");
  //struct tm timeinfo;
  //gmtime_r(&now, &timeinfo);
  //Serial.print("Current time: ");
  //Serial.print(asctime(&timeinfo));

  // Use WiFiClientSecure class to create TLS connection
  //Serial.print("Requesting URL: ");
  //Serial.println(url);


}

void loop() {


  WiFiClientSecure client;
  client.setInsecure();
  Serial.print("Connecting to ");
  Serial.println(github_host);

  if (!client.connect(github_host, github_port)) {
    Serial.println("Connection failed");
    return;
  }
 
  //Serial.printf("Using certificate: %s\n", cert_DigiCert_High_Assurance_EV_Root_CA);
  //client.setTrustAnchors(&cert);
  
  while(1){
  startMillis = millis();  //get the current "time" (actually the number of milliseconds since the program started)
  Serial.print("Start ");
  Serial.println(millis() -startMillis);

  client.connect(github_host, github_port);
  
  EndstopX = 1*int(EndstopXMin_Triggered) + 2*int(EndstopXMax_Triggered);     // 1-> Min Endstop, 2-> Max Endstop, 3 -> Error 
  if ((((millis()-lastFeed) > MillsRstTimeFeed)&&((millis()-lastFeed) < (MillsRstTimeFeed + 10000)))||(millis()<20000)){            // Futter Reset Flag jede 2h oder zu Beginn der ersten 20sek
    ResetFlag = 1;
    Fuetteranzahl = 0;
    
  }
  else {
    ResetFlag = 0;}
  url = "/esp/esp8266_update.php?check_LED=1&check_Stepper=1&EndstopX="+String(EndstopX)+"&Weg="+String(Weg)+"&ResetFlag="+String(ResetFlag);

  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + github_host + "\r\n" +
               "User-Agent: BuildFailureDetectorESP8266\r\n" +
               "Connection: close\r\n\r\n");
              //"Connection: keep-alive\r\n\r\n");
  
  while (client.connected()) {
 
    line += client.readStringUntil('\n');
  
  }

    Serial.println(line);
      
    if (line.indexOf("Data_Id1_is_2") >0 && (EndstopXMin_Triggered==false)) {
      digitalWrite(EnablePinDriverX, LOW);    //Treiber an
      digitalWrite(DirXPin, LOW);             //Links, Richtung setzen
      XMovement = 2;

    } else if (line.indexOf("Data_Id1_is_1") > 0 && (StepCounterX < MAX_STEPS_X)){          //Rechts, Posiiv in X-Richtung
      digitalWrite(EnablePinDriverX, LOW);
      digitalWrite(DirXPin, HIGH); 
      XMovement = 1;
    }
    else if (line.indexOf("Data_Id1_is_0" > 0)){
      digitalWrite(EnablePinDriverX, HIGH);
      XMovement = 0;
    
    }     
    //  FUTTERAUTOMAT Anforderung abfragen, erst nach 20 sek, damit WLAN Verbindung steht und SQL Datenbank zurückgesetzt wird
    if (line.indexOf("Data_Id3_is_1") >0 && (Fuetteranzahl < 1)) {            //1x Füttern angefragt
     futteraktiv = true;}
    if (line.indexOf("Data_Id3_is_2") >0 && (Fuetteranzahl < 2)) {            //2x Füttern angefragt
     futteraktiv = true;}
      while (futteraktiv){                                                    //Fütterung aktiv
      digitalWrite(EnablePinDriverX, LOW);                                    //Steppertreiber aktivieren
      delay(100);
      }             
  line = "";
  delay(100);
  }
  
}
